var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import '@nativescript/macos-node-api';
objc.import('AppKit');
let CustomAppDelegate;
export function setCustomDelegate(customAppDelegate) {
    CustomAppDelegate = customAppDelegate;
}
let AppDelegate = class AppDelegate extends NSObject {
    constructor() {
        super(...arguments);
        this.running = true;
        this.isActive = true;
    }
    applicationDidFinishLaunching(_notification) {
        NSApp.activateIgnoringOtherApps(false);
        NSApp.stop(this);
        // Allow users to customize the app's Touch Bar items
        NSApplication.sharedApplication.isAutomaticCustomizeTouchBarMenuItemEnabled = true;
        RunLoop();
    }
    applicationWillTerminate(_notification) {
        this.running = false;
    }
    openDocs(_id) {
        NSWorkspace.sharedWorkspace.openURL(NSURL.URLWithString('https://solidjs.com'));
    }
    openGithub(_id) {
        NSWorkspace.sharedWorkspace.openURL(NSURL.URLWithString('https://github.com/solidjs/solid'));
    }
    openDiscord(_id) {
        NSWorkspace.sharedWorkspace.openURL(NSURL.URLWithString('https://discord.com/invite/solidjs'));
    }
};
AppDelegate.ObjCProtocols = [NSApplicationDelegate];
AppDelegate.ObjCExposedMethods = {
    openDocs: { returns: interop.types.void, params: [interop.types.id] },
    openGithub: { returns: interop.types.void, params: [interop.types.id] },
    openDiscord: { returns: interop.types.void, params: [interop.types.id] },
};
AppDelegate = __decorate([
    NativeClass
], AppDelegate);
export function RunLoop() {
    let delay = 2;
    let lastEventTime = 0;
    const loop = () => {
        const event = NSApp.nextEventMatchingMaskUntilDateInModeDequeue(NSEventMask.Any, null, 'kCFRunLoopDefaultMode', true);
        const timeSinceLastEvent = Date.now() - lastEventTime;
        if (event != null) {
            NSApp.sendEvent(event);
            delay = timeSinceLastEvent < 32 ? 2 : 8;
            lastEventTime = Date.now();
        }
        else {
            delay = timeSinceLastEvent > 6000 ? 128 : timeSinceLastEvent > 4000 ? 64 : timeSinceLastEvent > 2000 ? 16 : 8;
        }
        if (NativeScriptApplication.delegate.running) {
            setTimeout(loop, NativeScriptApplication.ensure60FPS ? 8 : delay);
        }
    };
    setTimeout(loop, 0);
}
export class Application {
    static launch() {
        if (!(document.body instanceof HTMLElement)) {
            throw new Error('document.body instance of NSView');
        }
        Application.rootView = document.body;
        Application.rootView?.connectedCallback();
        if (NativeScriptApplication.window) {
            NativeScriptApplication.window.open();
        }
        else {
            throw new Error('Window is not initialized');
        }
        Application.application = NSApplication.sharedApplication;
        Application.delegate = typeof CustomAppDelegate !== 'undefined' ? CustomAppDelegate.new() : AppDelegate.new();
        Application.delegate.window = NativeScriptApplication.window.nativeView;
        Application.createMenu();
        NSApp.delegate = Application.delegate;
        NSApp.setActivationPolicy(NSApplicationActivationPolicy.Regular);
        NSApp.run();
    }
    static createMenu() {
        const menu = NSMenu.new();
        menu.delegate = Application.delegate;
        NSApp.mainMenu = menu;
        const appMenuItem = NSMenuItem.alloc().initWithTitleActionKeyEquivalent('Solid macOS', '', '');
        menu.addItem(appMenuItem);
        const submenu = NSMenu.new();
        appMenuItem.submenu = submenu;
        submenu.addItem(NSMenuItem.alloc().initWithTitleActionKeyEquivalent('Quit', 'terminate:', 'q'));
        const helpMenu = NSMenu.new();
        helpMenu.delegate = Application.delegate;
        NSApp.helpMenu = helpMenu;
        const helpMenuItem = NSMenuItem.alloc().initWithTitleActionKeyEquivalent('Help', '', '');
        menu.addItem(helpMenuItem);
        const helpSubmenu = NSMenu.new();
        helpMenuItem.submenu = helpSubmenu;
        helpSubmenu.addItem(NSMenuItem.alloc().initWithTitleActionKeyEquivalent('Open Docs', 'openDocs', 'i'));
        helpSubmenu.addItem(NSMenuItem.alloc().initWithTitleActionKeyEquivalent('Open Github', 'openGithub', 'g'));
        helpSubmenu.addItem(NSMenuItem.alloc().initWithTitleActionKeyEquivalent('Discord', 'openDiscord', 'd'));
    }
}
globalThis.NativeScriptApplication = Application;
//# sourceMappingURL=application.js.map