import '@nativescript/macos-node-api';
import { Layout } from '../layout/index.js';
import { Color } from './utils/color.js';
export function BackgroundColorNativeSet(style, _key, value) {
    if (!style.node.nativeView)
        return;
    const nativeView = style.node.nativeView;
    if (value) {
        nativeView.wantsLayer = true;
        nativeView.layer.backgroundColor = value;
    }
    else {
        nativeView.layer.backgroundColor = null;
    }
}
export const BackgroundColorStyle = {
    setNative: BackgroundColorNativeSet,
    defaultValue: undefined,
    converter: {
        toNative: (_key, value) => {
            return !value ? value : new Color(value).toNSColor().CGColor;
        },
    },
};
export function ZIndexNativeSet(style, _key, value) {
    if (!style.node.nativeView)
        return;
    const nativeView = style.node.nativeView;
    nativeView.wantsLayer = true;
    nativeView.layer.zPosition = value || 0;
}
export const ZIndexStyle = {
    setNative: ZIndexNativeSet,
    shouldLayout: true,
};
export function FontSizeNativeSet(style, _key, value) {
    if (!style.node.nativeView)
        return;
    const nativeView = style.node.nativeView;
    nativeView.font = nativeView.font.fontWithSize(value);
}
export const FontSizeStyle = {
    setNative: FontSizeNativeSet,
    shouldLayout: true,
};
export function FontStyleNativeSet(style, _key, value) {
    if (!style.node.nativeView)
        return;
    const nativeView = style.node.nativeView;
    const font = nativeView.font;
    switch (value) {
        case 'italic': {
            const italicFont = NSFontManager.sharedFontManager.convertFontToHaveTrait(font, NSFontTraitMask.Italic);
            nativeView.font = italicFont;
            break;
        }
        default:
            break;
    }
}
export const FontStyleStyle = {
    setNative: FontStyleNativeSet,
    shouldLayout: true,
};
export function ColorNativeSet(style, _key, value) {
    if (!style.node.nativeView)
        return;
    const nativeView = style.node.nativeView;
    nativeView.textColor = value;
}
export const ColorStyle = {
    setNative: ColorNativeSet,
    converter: {
        toNative: (_key, value) => (!value ? value : new Color(value).toNSColor()),
    },
};
export function BorderRadiusNativeSet(style, _key, value) {
    if (!style.node.nativeView)
        return;
    style.node.nativeView.wantsLayer = true;
    const nativeView = style.node.nativeView;
    nativeView.layer.cornerRadius = value;
}
export const BorderRadiusStyle = {
    setNative: BorderRadiusNativeSet,
    shouldLayout: true,
};
export function BorderColorSetNative(style, _key, value) {
    if (!style.node.nativeView)
        return;
    style.node.nativeView.wantsLayer = true;
    const nativeView = style.node.nativeView;
    nativeView.layer.borderColor = value;
}
export const BorderColorStyle = {
    setNative: BorderColorSetNative,
    converter: {
        toNative: (_key, value) => (!value ? value : new Color(value).toNSColor().CGColor),
    },
};
export function BorderWidthSetNative(style, key, value) {
    if (!style.node.nativeView)
        return;
    const nativeView = style.node.nativeView;
    nativeView.wantsLayer = true;
    nativeView.layer.borderWidth = value;
    // @ts-expect-error dynamic access of setters
    Layout.Setters[key](style.node.yogaNode, value);
}
export const BorderWidthStyle = {
    setNative: BorderWidthSetNative,
    shouldLayout: true,
};
export function OpacitySetNative(style, key, value) {
    if (!style.node.nativeView)
        return;
    const nativeView = style.node.nativeView;
    nativeView.alphaValue = value;
}
export const OpacityStyle = {
    setNative: OpacitySetNative,
};
//# sourceMappingURL=properties.js.map